#!/bin/bash

# parameter
threads=8
log=1
resultfile="TMscore.txt"
logfile="tmlog.txt"
exec="TMscore"

# author:li
# modified:2017-4-19 10:15

# reset savepid to 0
# mutilthread for calc
# mutilthread for saving
# with breakpoint supported
# with log supported
# some problems fixed
# more parameters
# saving refinement

function log()
{
    if [ $log -eq 1 ]; then
        date +"[%m/%d %H:%M:%S] $1" >> $logfile
    fi
}

function rmtmpdir()
{
    if [ -d ./tmpscore ]; then
        rm -rf ./tmpscore
    fi
}

function cleartmpdir()
{
    rmtmpdir
    mkdir ./tmpscore
}

function clearsavetmp()
{
    if [ -d ./saving ]; then
        rm -rf ./saving
    fi
}

function clearresult()
{
    if [ -f ./$resultfile ]; then
        rm ./$resultfile
    fi
}

function clearlog()
{
    if [ -f ./$logfile ]; then
        rm ./$logfile
    fi
}

function listpdb()
{
    ls *.pdb > pdblist.txt
    array=($(awk '{print $0}' pdblist.txt))
    length=${#array[*]}
}

function savebp()
{
    echo $saving > breakpoint.calc
    log saved-$saving
}

function checkbp()
{
    if [ -f ./breakpoint.calc ]; then
        read -p "It looks u break the calc last time,will u continue?(y/n)" bp
        if [[ $bp == "y" ]]; then
            # to continue
            bp=$(cat breakpoint.calc)
            let bp++
            if [ $bp -ge $length ]; then
                echo finished.
                rmtmpdir
                exit
            fi
        else
            # to restart
            bp=0
            rm breakpoint.calc
            clearresult
        fi
        # log the operation
        if [ $bp -eq 0 ]; then
            log recalc
            echo recalc
        else
            log continue-$bp
            echo "continue-$bp"
        fi
        tput sc
    else
        # without breakpoint
        clearlog
    fi
}

function init()
{
    if [ $threads -lt 1 ]; then
        echo 'threads shall be larger than(equal to) 1.'
        exit
    fi

    clearsavetmp
    cleartmpdir
    calcpid=()
    savepid=0
    idx=0
    bp=0
    tput sc
}

function waitcalc()
{
    wait ${calcpid[@]}
    calcpid=()
    idx=0
}

function waitsave()
{
    tput rc
    tput ed
    if [ $savepid -ne 0 ]; then
        echo -n 'waitting for last saving...'
        wait $savepid
        savepid=0
    fi
}

function calcthread()
{
    $exec ${array[i]} ${array[j]} > ./tmpscore/$i-$j.txt
}

function savingthread()
{
    for((k=0;k<length;k++))
    do
        cat ./saving/$saving-$k.txt | grep "TM-score    = " | awk '{print $3}' >> ./saving/$resultfile.saving
    done
    # if breaking in cat, something to fix
    cat ./saving/$resultfile.saving >> $resultfile && savebp

    clearsavetmp
}

function newcalc()
{
    calcthread &
    calcpid[idx]=$!
    let idx++
    tput rc
    tput ed
    echo -n $i,$j
}

function save2result()
{
    waitsave

    tput rc
    tput ed
    echo -n $i,saving...
    mv ./tmpscore ./saving
    cleartmpdir

    saving=$i
    log saving-$saving
    savingthread &
    savepid=$!
}

function docalc()
{
    for((i=bp;i<length;i++))
    do
        for((j=0;j<length;j++))
        do
            if [ $idx -ge $threads ]; then
                waitcalc
            fi
            newcalc
        done
        waitcalc
        log calced-$i
        save2result
    done
    echo;
}

function main()
{
    init
    listpdb
    checkbp
    docalc
    waitsave
    rmtmpdir
    log finished.
    echo finished.
}

main # run main
